# Reading Machine

A reading machine can be thought of as a kind of deterministic finite automaton where :
* The machine is made of *states*, *transitions* and a *strategy*.
* The machine works on a *configuration*, representing the input text to be annotated and its annotations so far.
* A *transition* is an endofunction in the domain of *configurations*, making a small change like adding a single annotation.
* The *strategy* is a function that takes the current *state* and the chosen *transition* as inputs, and returns the new *state*.
* The machine contains a classifier that is trained to predict the next *transition* to take, given the current *configuration*.
* At each step, and until the *configuration* is final :
	1. The classifier will predict the next *transition* to take.
	2. This *transition* will be applied to the current *configuration*, thus yielding the new *configuration*.
	3. The *strategy* will determine the new *state*.

## Configuration :

A configuration is the current state of the analysis, it is made of the input text along with all of the annotations predicted so far.\
It is said to be final when all the input text has been processed.

## File format :

A reading machine is defined in a `.rm` file (or given as argument to `macaon train`).\
Here is an example of a Reading Machine doing tokenization, POS tagging, Morphological tagging, dependency parsing and sentence segmentation in a sequential fashion :

```
Name : Tokenizer, Tagger, Morpho, Lemmatizer, Parser and Segmenter Machine
Classifier : tokeparser
{
  Transitions : {tokenizer,data/tokenizer.ts tagger,data/tagger.ts morpho,data/morpho_whole.ts lemmatizer_rules,data/lemmatizer_rules.ts lemmatizer_case,data/lemmatizer_case.ts parser,data/parser_eager_rel_strict.ts segmenter,data/segmenter.ts}
  LossMultiplier : {}
	Network type : Modular
  Contextual : Window{-10 10} Columns{FORM} LSTM{1 1 0 1} In{64} Out{128} w2v{FORM,data/FORM.w2v} Targets{b.-3 b.-2 b.-1 b.0 b.1 b.2 s.0 s.1 s.2 b.0.0 s.0.0 s.0.-1 s.1.0 s.1.-1 s.2.0 s.2.-1}
  Context : Targets{b.-3 b.-2 b.-1 b.0 b.1 b.2 s.0 s.1 s.2 b.0.0 s.0.0 s.0.-1 s.1.0 s.1.-1 s.2.0 s.2.-1} Columns{EOS ID UPOS FEATS DEPREL} LSTM{1 1 0 1} In{64} Out{64} w2v{}
  Focused : Column{prefix3:FORM} NbElem{3} Buffer{0} Stack{} LSTM{1 1 0 1} In{64} Out{64} w2v{}
  Focused : Column{suffix3:FORM} NbElem{3} Buffer{0} Stack{} LSTM{1 1 0 1} In{64} Out{64} w2v{}
	RawInput : Left{5} Right{10} LSTM{1 1 0.0 1} In{32} Out{32}
  History : NbElem{10} LSTM{1 1 0 1} In{32} Out{32}
	SplitTrans : LSTM{1 1 0.0 1} In{64} Out{64}
	InputDropout : 0.3
	MLP : {1600 0.3 1600 0.3}
	End
  Optimizer : Adagrad {0.01 0.000001 0 0.0000000001}
  Type : classification
  Loss : crossentropy
}
Splitwords : data/splitwords.ts
Predictions : ID FORM UPOS FEATS LEMMA HEAD DEPREL EOS
Strategy
{
	Block : End{cannotMove}
	tokenizer tokenizer ENDWORD 1
	tokenizer tokenizer SPLIT 1
	tokenizer tokenizer * 0
	Block : End{cannotMove}
	tagger tagger * 1
	Block : End{cannotMove}
	morpho morpho * 1
	Block : End{cannotMove}
	lemmatizer_rules lemmatizer_case * 0
  lemmatizer_case lemmatizer_rules * 1
	Block : End{cannotMove}
	parser segmenter eager_SHIFT 0
	parser segmenter eager_RIGHT_rel 0
	parser parser * 0
	segmenter parser * 1
}
```

This format is composed of several parts :
* Name : The name of your machine.
* Classifier : The name of your classifier, followed by its definition between braces. There can be as many classifiers as you want. In the above example, we only define one. See [Classifier](classifier.md).
* Splitwords : [Transition Set](transitionSet.md) file that contains transitions for multi-words tokenization.\
It is only mandatory if the machine performs tokenization. This file is automatically generated by `train.sh`.
* Predictions : Names of the columns that are predicted by your machine.
* Strategy, followed by its definition between braces. See [Strategy](strategy.md).

[Back to main page](../README.md)
